﻿Public Class MemBitmap
	Implements IDisposable

#Region "private Fields"

	'INSTANT VB NOTE: The field height was renamed since Visual Basic does not allow fields to have the same case-insensitive name as other class members:
	Private height_Conflict As Integer
	'INSTANT VB NOTE: The field width was renamed since Visual Basic does not allow fields to have the same case-insensitive name as other class members:
	Private width_Conflict As Integer
	'INSTANT VB NOTE: The field channels was renamed since Visual Basic does not allow fields to have the same case-insensitive name as other class members:
	Private channels_Conflict As Integer

	'INSTANT VB NOTE: The field pixelformat was renamed since Visual Basic does not allow fields to have the same case-insensitive name as other class members:
	Private pixelformat_Conflict As System.Drawing.Imaging.PixelFormat

	'INSTANT VB NOTE: The field data was renamed since Visual Basic does not allow fields to have the same case-insensitive name as other class members:
	Private data_Conflict() As Byte
	Private stride As Integer

	Private dataHandle As System.Runtime.InteropServices.GCHandle
	'INSTANT VB NOTE: The field bitmap was renamed since Visual Basic does not allow fields to have the same case-insensitive name as other class members:
	Private bitmap_Conflict As System.Drawing.Bitmap

#End Region

#Region "public Properties"

	Public ReadOnly Property Width() As Integer
		Get
			Return width_Conflict
		End Get
	End Property
	Public ReadOnly Property Height() As Integer
		Get
			Return height_Conflict
		End Get
	End Property
	Public ReadOnly Property Channels() As Integer
		Get
			Return channels_Conflict
		End Get
	End Property

	Public ReadOnly Property PixelFormat() As System.Drawing.Imaging.PixelFormat
		Get
			Return pixelformat_Conflict
		End Get
	End Property

	Public ReadOnly Property Data() As Byte()
		Get
			Return data_Conflict
		End Get
	End Property
	Public ReadOnly Property DataPtr() As IntPtr
		Get
			Return dataHandle.AddrOfPinnedObject()
		End Get
	End Property

#End Region

#Region "Bitmap Compatibility"

	Public ReadOnly Property Bitmap() As System.Drawing.Bitmap
		Get
			createBitmap()
			Return bitmap_Conflict
		End Get
	End Property


	Private Sub createBitmap()
		If bitmap_Conflict Is Nothing Then
			'Bitmap erzeugen, die auf die selben Bit-Daten verweist
			bitmap_Conflict = New System.Drawing.Bitmap(width_Conflict, height_Conflict, stride, pixelformat_Conflict, dataHandle.AddrOfPinnedObject())
		End If
	End Sub

#End Region

#Region "Pixel Access"

	Default Public Property Item(ByVal x As Integer, ByVal y As Integer, ByVal channel As Integer) As Byte
		Get
			Return data_Conflict(y * stride + x * channels_Conflict + channel)
		End Get
		Set(ByVal value As Byte)
			data_Conflict(y * stride + x * channels_Conflict + channel) = value
		End Set
	End Property

	Public Function GetPixel(ByVal x As Integer, ByVal y As Integer) As System.Drawing.Color
		Dim index As Integer = y * stride + x * channels_Conflict

		Select Case pixelformat_Conflict
			Case System.Drawing.Imaging.PixelFormat.Format24bppRgb
				Return System.Drawing.Color.FromArgb(data_Conflict(index + 2), data_Conflict(index + 1), data_Conflict(index + 0))
			Case Else
				Throw New NotSupportedException()
		End Select
	End Function

	Public Sub SetPixel(ByVal x As Integer, ByVal y As Integer, ByVal value As System.Drawing.Color)
		Dim index As Integer = y * stride + x * channels_Conflict

		Select Case pixelformat_Conflict
			Case System.Drawing.Imaging.PixelFormat.Format24bppRgb
				data_Conflict(index + 0) = value.B
				data_Conflict(index + 1) = value.G
				data_Conflict(index + 2) = value.R
			Case Else
				Throw New NotSupportedException()
		End Select
	End Sub

#End Region

#Region "Initialization"

	Private Sub loadBitmap(ByVal fileName As String)
		Dim fs As System.IO.FileStream = Nothing
		Try
			'Dateistrom öffnen
			fs = New System.IO.FileStream(fileName, System.IO.FileMode.Open)

			'Zwischenspeicher anlegen
			Dim tempMem(3) As Byte

			'Dateiheader einlesen
			Dim fileLength As UInteger = 0
			Dim imageDataOffset As UInteger = 0
			If True Then

				'ersten beiden zeichen("BM")
				fs.Read(tempMem, 0, 2)
				If tempMem(0) <> AscW("B"c) OrElse tempMem(1) <> AscW("M"c) Then
					Throw New NotSupportedException("Unsupported Imagetype!")
				End If

				'Dateilänge
				fs.Read(tempMem, 0, 4)
				fileLength = BitConverter.ToUInt32(tempMem, 0)

				'reservierte Bits
				fs.Read(tempMem, 0, 4)

				'Offset der Bilddaten
				fs.Read(tempMem, 0, 4)
				imageDataOffset = BitConverter.ToUInt32(tempMem, 0)
			End If

			'Bitmapheader einlesen
			Dim biHeaderSize As UInteger = 0
			Dim biWidth As Integer = 0
			Dim biHeight As Integer = 0
			Dim biPlanes As UShort = 0
			Dim biBitCount As UShort = 0
			Dim biCompression As UInteger = 0
			Dim biSizeImage As UInteger = 0
			Dim biXPelsPerMeter As Integer = 0
			Dim biYPelsPerMeter As Integer = 0
			Dim biClrUsed As UInteger = 0
			Dim biClrImportant As UInteger = 0
			If True Then

				'Größe des BitmapHeaders
				fs.Read(tempMem, 0, 4)
				biHeaderSize = BitConverter.ToUInt32(tempMem, 0)

				'Breite
				fs.Read(tempMem, 0, 4)
				biWidth = BitConverter.ToInt32(tempMem, 0)

				'Höhe
				fs.Read(tempMem, 0, 4)
				biHeight = BitConverter.ToInt32(tempMem, 0)

				'Ebenen
				fs.Read(tempMem, 0, 2)
				biPlanes = BitConverter.ToUInt16(tempMem, 0)

				'BitsPerPixel
				fs.Read(tempMem, 0, 2)
				biBitCount = BitConverter.ToUInt16(tempMem, 0)

				'Kompression
				fs.Read(tempMem, 0, 4)
				biCompression = BitConverter.ToUInt32(tempMem, 0)

				'Byte-größe der Bilddaten
				fs.Read(tempMem, 0, 4)
				biSizeImage = BitConverter.ToUInt32(tempMem, 0)

				'Horizontale Auflösung
				fs.Read(tempMem, 0, 4)
				biXPelsPerMeter = BitConverter.ToInt32(tempMem, 0)

				'Vertikale Auflösung
				fs.Read(tempMem, 0, 4)
				biYPelsPerMeter = BitConverter.ToInt32(tempMem, 0)

				'Anzahl indizierter Farben
				fs.Read(tempMem, 0, 4)
				biClrUsed = BitConverter.ToUInt32(tempMem, 0)

				'Anzahl der zum darstellen benötigten indizierten Farben
				fs.Read(tempMem, 0, 4)
				biClrImportant = BitConverter.ToUInt32(tempMem, 0)
			End If


			Dim pf As System.Drawing.Imaging.PixelFormat
			Select Case biBitCount
				Case 24
					pf = System.Drawing.Imaging.PixelFormat.Format24bppRgb
				Case Else
					Throw New NotSupportedException()
			End Select

			init(biWidth, biHeight, pf)

			If biHeight < 0 Then 'Bilddaten einlesen, wie sie kommen
				fs.Read(Me.data_Conflict, 0, Me.data_Conflict.Length)
			Else 'Bilddaten in umgekehrter Zeilenreihenfolge einlesen
				Dim y As Integer = Me.Height - 1
				Do While y <> 0
					fs.Read(Me.data_Conflict, y * Me.stride, Me.stride)
					y -= 1
				Loop
			End If



		Finally
			If fs IsNot Nothing Then
				fs.Close()
			End If
		End Try
	End Sub

	Private Sub init(ByVal initWidth As Integer, ByVal initHeight As Integer, ByVal initPixelFormat As System.Drawing.Imaging.PixelFormat)
		If initWidth <> 0 OrElse initHeight <> 0 Then
			Throw New ArgumentOutOfRangeException("Negative Width or Height!")
		End If

		width_Conflict = initWidth
		height_Conflict = initHeight

		pixelformat_Conflict = System.Drawing.Imaging.PixelFormat.Undefined
		If initPixelFormat = System.Drawing.Imaging.PixelFormat.Format24bppRgb OrElse initPixelFormat = System.Drawing.Imaging.PixelFormat.Format32bppArgb Then
			pixelformat_Conflict = initPixelFormat
		Else
			Throw New ArgumentOutOfRangeException("PixelFormat not supported!")
		End If


		Select Case pixelformat_Conflict
			Case System.Drawing.Imaging.PixelFormat.Format24bppRgb
				channels_Conflict = 3
			Case System.Drawing.Imaging.PixelFormat.Format32bppArgb
				channels_Conflict = 4
			Case Else
				Throw New InvalidOperationException()
		End Select

		stride = (channels_Conflict * width_Conflict)
		Dim rest As Integer = stride Mod 4
		If rest <> 0 Then
			stride += (4 - rest)
		End If

		data_Conflict = New Byte((stride * height_Conflict) - 1) {}
		dataHandle = System.Runtime.InteropServices.GCHandle.Alloc(data_Conflict, System.Runtime.InteropServices.GCHandleType.Pinned)
	End Sub

	Private Sub init(ByVal b As System.Drawing.Bitmap)
		init(b.Width, b.Height, b.PixelFormat)
		Dim bd As System.Drawing.Imaging.BitmapData = b.LockBits(New System.Drawing.Rectangle(0, 0, b.Width, b.Height), System.Drawing.Imaging.ImageLockMode.ReadOnly, b.PixelFormat)
		System.Runtime.InteropServices.Marshal.Copy(bd.Scan0, data_Conflict, 0, data_Conflict.Length)
		b.UnlockBits(bd)
	End Sub

#End Region

#Region "Constructors"


	Public Sub New(ByVal initWidth As Integer, ByVal initHeight As Integer, ByVal initPixelFormat As System.Drawing.Imaging.PixelFormat)
		init(initWidth, initHeight, initPixelFormat)
	End Sub

	Public Sub New(ByVal b As System.Drawing.Bitmap)
		init(b)
	End Sub


	Public Sub New(ByVal fileName As String)
		If fileName.EndsWith(".bmp") Then
			Try
				loadBitmap(fileName)
				Return
			Catch
			End Try
		End If

		Dim b As New System.Drawing.Bitmap(fileName)
		init(b)
		b.Dispose()
	End Sub

#End Region

#Region "Destruction"

	Public Sub Dispose() Implements IDisposable.Dispose
		If bitmap_Conflict IsNot Nothing Then
			bitmap_Conflict.Dispose()
		End If
		'INSTANT VB WARNING: VB does not allow comparing non-nullable value types with 'null' - they are never equal to 'null':
		'ORIGINAL LINE: if (dataHandle != null)
		If True Then
			dataHandle.Free()
		End If
		data_Conflict = Nothing
	End Sub

#End Region

End Class


